/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import java.util.Locale;


/**
 * <p>
 * Data class for storing the information in DOCUMENT table.
 * This class is added to simplify arguments passing.
 * </p>
 *
 * <p>
 * Note, one instance of Document class represents a corresponding record in DOCUMENT table.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 *
 */
final class Document {
    /**
     * <p>
     * Represents the document index id for a DocumentIndex instance.
     * This field is related to DOCUMENT_ID column in DOCUMENT table.
     * </p>
     */
    private final String documentId;

    /**
     * <p>
     * Represents the Locale information for a DocumentIndex instance.
     * This field is related to LOCALE column in DOCUMENT table.
     * </p>
     */
    private final Locale locale;

    /**
     * <p>
     * Represents the class name for a DocumentIndex instance.
     * This field is related to CLASS_NAME column in DOCUMENT table.
     * </p>
     */
    private final String className;

    /**
     * <p>
     * Represents the use count for a DocumentIndex instance.
     * This field is related to USE_COUNT column in DOCUMENT table.
     * </p>
     */
    private final int useCount;

    /**
     * <p>
     * Represents the source identity for a DocumentIndex instance.
     * This field is related to SOURCE_IDENTITY column in DOCUMENT table.
     * </p>
     */
    private final Object sourceIdentity;

    /**
     * <p>
     * Create a new Document instance with pass-in arguments.
     * </p>
     *
     * <p>
     * Note, it's package private.
     * </p>
     *
     * @param documentId the document index id for a DocumentIndex instance.
     * @param locale the Locale information for a DocumentIndex instance.
     * @param sourceIdentity the source identity for a DocumentIndex instance.
     * @param className the class name for a DocumentIndex instance.
     * @param useCount the use count for a DocumentIndex instance.
     */
    Document(String documentId, Locale locale, Object sourceIdentity,
        String className, int useCount) {
        this.documentId = documentId;
        this.locale = locale;
        this.sourceIdentity = sourceIdentity;
        this.className = className;
        this.useCount = useCount;
    }

    /**
     * <p>
     * Returns the use count for a DocumentIndex instance.
     * </p>
     *
     * <p>
     * Note, the returned value is related to USE_COUNT column in DOCUMENT table.
     * </p>
     *
     * @return the use count for a DocumentIndex instance.
     */
    int getUseCount() {
        return useCount;
    }

    /**
     * <p>
     * Returns the class name for a DocumentIndex instance.
     * </p>
     *
     * <p>
     * Note, the returned value is related to CLASS_NAME column in DOCUMENT table.
     * </p>
     *
     * @return the class name for a DocumentIndex instance.
     */
    String getClassName() {
        return className;
    }

    /**
     * <p>
     * Returns the Locale information for a DocumentIndex instance.
     * </p>
     *
     * <p>
     * Note, the returned value is related to LOCALE column in DOCUMENT table.
     * </p>
     *
     * @return the Locale information for a DocumentIndex instance.
     */
    Locale getLocale() {
        return locale;
    }

    /**
     * <p>
     * Returns the source identity for a DocumentIndex instance.
     * </p>
     *
     * <p>
     * Note, the returned value is related to SOURCE_IDENTITY column in DOCUMENT table.
     * </p>
     *
     * @return the source identity for a DocumentIndex instance.
     */
    Object getSourceIdentity() {
        return sourceIdentity;
    }

    /**
     * <p>
     * Returns the document index id for a DocumentIndex instance.
     * </p>
     *
     * <p>
     * Note, the returned value is related to DOCUMENT_ID column in DOCUMENT table.
     * </p>
     *
     * @return the document index id for a DocumentIndex instance.
     */
    String getDocumentId() {
        return documentId;
    }
}
