/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.db.connectionfactory.DBConnectionFactory;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.IndexManager;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.TestHelper;
import com.topcoder.document.index.wordsource.WordSource;
import com.topcoder.document.index.wordsource.WordSourceId;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.util.Set;


/**
 * <p>
 * Unit test cases for FastAccessDBIndexPersistence.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class FastAccessDBIndexPersistenceTests extends TestCase {
    /**
     * <p>
     * CollectionIndex Id constant for testing.
     * </p>
     */
    private static final String TEST_COLLECTION_ID = "478451452";

    /**
     * <p>
     * FastAccessDBIndexPersistence instance for testing.
     * </p>
     */
    private FastAccessDBIndexPersistence persistence;

    /**
     * <p>
     * IndexManager instance for testing.
     * </p>
     */
    private IndexManager im;

    /**
     * <p>
     * Setup test environment.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    protected void setUp() throws Exception {
        //set up necessary namespaces
        TestHelper.loadSingleXMLConfig(TestHelper.FAST_ACCESS_NAMESPACE,
            TestHelper.FAST_ACCESS_CONFIGFILE);
        TestHelper.loadMultipleXMLConfig(TestHelper.DB_FACTORY_NAMESPACE,
            TestHelper.DB_FACTORY_CONFIGFILE);

        persistence = new FastAccessDBIndexPersistence(TestHelper.FAST_ACCESS_NAMESPACE);
        im = new IndexManager(persistence);
    }

    /**
     * <p>
     * Tears down test environment.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    protected void tearDown() throws Exception {
        //clear all the data in database
        DBConnectionFactory dbFactory = TestHelper.getDBFactory();
        TestHelper.clearFastSolutionTables(dbFactory.createConnection(
                TestHelper.FAST_ACCESS_CONNECTION_NAME));

        // clear namespaces
        TestHelper.clearConfigFile(TestHelper.FAST_ACCESS_NAMESPACE);
        TestHelper.clearConfigFile(TestHelper.DB_FACTORY_NAMESPACE);

        persistence = null;
        im = null;
    }

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(FastAccessDBIndexPersistenceTests.class);
    }

    /**
     * <p>
     * Tests ctor FastAccessDBIndexPersistence#FastAccessDBIndexPersistence(DBConnectionFactory,String) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor1() throws Exception {
        assertNotNull("Failed to create FastAccessDBIndexPersistence instance.",
            new FastAccessDBIndexPersistence(TestHelper.getDBFactory(),
                TestHelper.FAST_ACCESS_CONNECTION_NAME));
    }

    /**
     * <p>
     * Tests ctor FastAccessDBIndexPersistence#FastAccessDBIndexPersistence(DBConnectionFactory,String) for failure.
     * It tests the case that when connectionFactory is null and expects IllegalArgumentException.
     * </p>
     */
    public void testCtor1_NullConnectionFactory() {
        try {
            new FastAccessDBIndexPersistence(null,
                TestHelper.FAST_ACCESS_CONNECTION_NAME);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor FastAccessDBIndexPersistence#FastAccessDBIndexPersistence(DBConnectionFactory,String) for accuracy.
     * It tests the case that when connectionName is null and expects it success.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor1_NullConnectionName() throws Exception {
        assertNotNull("Failed to create FastAccessDBIndexPersistence instance with null connection name.",
            new FastAccessDBIndexPersistence(TestHelper.getDBFactory(), null));
    }

    /**
     * <p>
     * Tests ctor FastAccessDBIndexPersistence#FastAccessDBIndexPersistence(DBConnectionFactory,String) for accuracy.
     * It tests the case that when connectionName is empty and expects it success.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor1_EmptyConnectionName() throws Exception {
        assertNotNull("Failed to create FastAccessDBIndexPersistence instance with empty connection name.",
            new FastAccessDBIndexPersistence(TestHelper.getDBFactory(), " "));
    }

    /**
     * <p>
     * Tests ctor FastAccessDBIndexPersistence#FastAccessDBIndexPersistence(String) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor2() throws Exception {
        assertNotNull("Failed to create FastAccessDBIndexPersistence instance.",
            new FastAccessDBIndexPersistence(TestHelper.FAST_ACCESS_NAMESPACE));
    }

    /**
     * <p>
     * Tests ctor FastAccessDBIndexPersistence#FastAccessDBIndexPersistence(String) for failure.
     * It tests the case that when namespace is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor2_NullNamespace() throws Exception {
        try {
            new FastAccessDBIndexPersistence(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor FastAccessDBIndexPersistence#FastAccessDBIndexPersistence(String) for failure.
     * It tests the case that when namespace is empty and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor2_EmptyNamespace() throws Exception {
        try {
            new FastAccessDBIndexPersistence(" ");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#addDocumentIndex(DocumentIndex) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testAddDocumentIndex() throws Exception {
        DocumentIndex docIndex = new DocumentIndex(TestHelper.getMediumWordSource());
        persistence.addDocumentIndex(docIndex);

        DocumentIndex gettedIndex = persistence.getDocumentIndex(docIndex.getWordSourceId());

        assertEquals("Failed to add document correctly.", gettedIndex.getWords().size(), docIndex.getWords().size());
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#addDocumentIndex(DocumentIndex) for failure.
     * It tests the case that when documentIndex is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testAddDocumentIndex_NullDocumentIndex()
        throws Exception {
        try {
            persistence.addDocumentIndex(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#addDocumentIndex(DocumentIndex) for failure.
     * It tests the case when trying to add a persisted document and expects IndexPersistenceException.
     * </p>
     *
     * @throws Exception to JUit.
     */
    public void testAddDocumentIndex_IndexPersistenceException()
        throws Exception {
        try {
            DocumentIndex documentIndex = im.createDocumentIndex(TestHelper.getSimpleWordSource());

            persistence.addDocumentIndex(documentIndex);

            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#getDocumentIndex(WordSourceId) for accuracy.
     * It test the case when the wordSourceId hasn't been persisted and expects null.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetDocumentIndex() throws Exception {
        DocumentIndex index = persistence.getDocumentIndex(TestHelper.getSimpleWordSource()
                                                                     .getId());

        assertNull("The document index should not exist in persistence.", index);
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#getDocumentIndex(WordSourceId) for failure.
     * It tests the case that when wordSourceId is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetDocumentIndex_NullWordSourceId()
        throws Exception {
        try {
            persistence.getDocumentIndex(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#removeDocumentIndex(WordSourceId) for accuracy.
     * Expects the indexed document removed successfully.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testRemoveDocumentIndex() throws Exception {
        DocumentIndex docIndex = new DocumentIndex(TestHelper.getMediumWordSource());
        persistence.addDocumentIndex(docIndex);

        persistence.removeDocumentIndex(docIndex.getWordSourceId());

        assertNull("The document index should be deleted.",
            persistence.getDocumentIndex(docIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#removeDocumentIndex(WordSourceId) for failure.
     * It tests the case that when wordSourceId is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testRemoveDocumentIndex_NullWordSourceId()
        throws Exception {
        try {
            persistence.removeDocumentIndex(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#removeDocumentIndex(WordSourceId) for failure.
     * It tests the case when trying to remove a not persisted WordSourceId and expects IndexPersistenceException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testRemoveDocumentIndex_IndexPersistenceException()
        throws Exception {
        try {
            persistence.removeDocumentIndex(TestHelper.getSimpleWordSource()
                                                      .getId());
            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#addCollectionIndex(CollectionIndex) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testAddCollectionIndex() throws Exception {
        im.createCollectionIndex(TEST_COLLECTION_ID);

        CollectionIndex collectionIndex = persistence.getCollectionIndex(TEST_COLLECTION_ID);
        assertEquals("Expects equal collection id.", TEST_COLLECTION_ID,
            collectionIndex.getId());
        assertEquals("Expects zero document contained.", 0,
            collectionIndex.getAllDocumentIds().size());
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#addCollectionIndex(CollectionIndex) for failure.
     * It tests the case that when collectionIndex is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testAddCollectionIndex_NullCollectionIndex()
        throws Exception {
        try {
            persistence.addCollectionIndex(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#addCollectionIndex(CollectionIndex) for failure.
     * It tests the case when trying to add a persisted WordSourceId and expects IndexPersistenceException.
     * </p>
     */
    public void testAddCollectionIndex_IndexPersistenceException() {
        try {
            CollectionIndex ci = im.createCollectionIndex(TEST_COLLECTION_ID);

            persistence.addCollectionIndex(ci);

            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#getCollectionIndex(String) for accuracy.
     * It tests the case when trying to get a not persisted document collection and expects null.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetCollectionIndex() throws Exception {
        CollectionIndex collectionIndex = persistence.getCollectionIndex(TEST_COLLECTION_ID);
        assertNull("The collection index should not exist in persistence.",
            collectionIndex);
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#getCollectionIndex(String) for failure.
     * It tests the case that when collectionId is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetCollectionIndex_NullCollectionId()
        throws Exception {
        try {
            persistence.getCollectionIndex(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#getCollectionIndex(String) for failure.
     * It tests the case that when collectionId is empty and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetCollectionIndex_EmptyCollectionId()
        throws Exception {
        try {
            persistence.getCollectionIndex(" ");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#removeCollectionIndex(String) for accuracy.
     * Expects the indexed document collection removed successfully.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testRemoveCollectionIndex() throws Exception {
        //set up the database environment.
        im.createCollectionIndex(TEST_COLLECTION_ID);

        //remove it
        persistence.removeCollectionIndex(TEST_COLLECTION_ID);

        CollectionIndex collectionIndex = persistence.getCollectionIndex(TEST_COLLECTION_ID);
        assertNull("The collection index should be removed.", collectionIndex);
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#removeCollectionIndex(String) for failure.
     * It tests the case that when collectionId is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testRemoveCollectionIndex_NullCollectionId()
        throws Exception {
        try {
            persistence.removeCollectionIndex(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#removeCollectionIndex(String) for failure.
     * It tests the case that when collectionId is empty and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testRemoveCollectionIndex_EmptyCollectionId()
        throws Exception {
        try {
            persistence.removeCollectionIndex(" ");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#removeCollectionIndex(String) for failure.
     * It tests the case when trying to remove not persisted document collection and expects
     * IndexPersistenceException.
     * </p>
     */
    public void testRemoveCollectionIndex_IndexPersistenceException() {
        try {
            persistence.removeCollectionIndex("0001");
            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#updateCollectionIndex(CollectionIndex) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testUpdateCollectionIndex() throws Exception {
        //set up the database environment.
        CollectionIndex collectionIndex = im.createCollectionIndex(TEST_COLLECTION_ID);

        //add a new document
        collectionIndex.addDocument(TestHelper.getSimpleWordSource());

        collectionIndex = persistence.getCollectionIndex(TEST_COLLECTION_ID);
        assertEquals("Expects equal collection id.", TEST_COLLECTION_ID,
            collectionIndex.getId());
        assertEquals("Expects one document contained.", 1,
            collectionIndex.getAllDocumentIds().size());
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#updateCollectionIndex(CollectionIndex) for failure.
     * It tests the case that when collectionIndex is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testUpdateCollectionIndex_NullCollectionIndex()
        throws Exception {
        try {
            persistence.updateCollectionIndex(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#updateCollectionIndex(CollectionIndex) for failure.
     * It tests the case when trying to update not persisted document collection and expects
     * IndexPersistenceException.
     * </p>
     */
    public void testUpdateCollectionIndex_IndexPersistenceException() {
        try {
            CollectionIndex ci = new CollectionIndex(im, TEST_COLLECTION_ID);
            persistence.updateCollectionIndex(ci);
            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#increaseDocumentUseCount(WordSourceId) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testIncreaseDocumentUseCount() throws Exception {
        WordSource source = TestHelper.getSimpleWordSource();
        im.createDocumentIndex(source);

        persistence.increaseDocumentUseCount(source.getId());

        int useCount = persistence.getDocumentUseCount(source.getId());
        assertEquals("Expects the value of use count is one.", 1, useCount);
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#increaseDocumentUseCount(WordSourceId) for failure.
     * It tests the case that when wordSourceId is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testIncreaseDocumentUseCount_NullWordSourceId()
        throws Exception {
        try {
            persistence.increaseDocumentUseCount(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#increaseDocumentUseCount(WordSourceId) for failure.
     * It tests the case when trying to increase the use count of a not persisted document and expects
     * IndexPersistenceException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testIncreaseDocumentUseCount_IndexPersistenceException()
        throws Exception {
        try {
            persistence.increaseDocumentUseCount(TestHelper.getSimpleWordSource()
                                                           .getId());
            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#decreaseDocumentUseCount(WordSourceId) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDecreaseDocumentUseCount() throws Exception {
        WordSource source = TestHelper.getSimpleWordSource();
        WordSourceId id = source.getId();
        im.createDocumentIndex(source);
        persistence.increaseDocumentUseCount(id);

        persistence.decreaseDocumentUseCount(id);

        int useCount = persistence.getDocumentUseCount(id);
        assertEquals("Expects the value of use count is zero.", 0, useCount);
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#decreaseDocumentUseCount(WordSourceId) for failure.
     * It tests the case that when wordSourceId is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDecreaseDocumentUseCount_NullWordSourceId()
        throws Exception {
        try {
            persistence.decreaseDocumentUseCount(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#decreaseDocumentUseCount(WordSourceId) for failure.
     * It tests the case when trying to decrease the use count of a not persisted document and expects
     * IndexPersistenceException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDecreaseDocumentUseCount_IndexPersistenceException()
        throws Exception {
        try {
            persistence.decreaseDocumentUseCount(TestHelper.getSimpleWordSource()
                                                           .getId());
            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#getDocumentUseCount(WordSourceId) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetDocumentUseCount() throws Exception {
        WordSource source = TestHelper.getSimpleWordSource();
        im.createDocumentIndex(source);

        int useCount = persistence.getDocumentUseCount(source.getId());

        assertEquals("Expects the value of use count is zero.", 0, useCount);
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#getDocumentUseCount(WordSourceId) for failure.
     * It tests the case that when wordSourceId is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetDocumentUseCount_NullWordSourceId()
        throws Exception {
        try {
            persistence.getDocumentUseCount(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#getDocumentUseCount(WordSourceId) for failure.
     * It tests the case when trying to get the use count of a not persisted document and expects
     * IndexPersistenceException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetDocumentUseCount_IndexPersistenceException()
        throws Exception {
        try {
            persistence.getDocumentUseCount(TestHelper.getSimpleWordSource()
                                                           .getId());
            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests FastAccessDBIndexPersistence#getIndexedDocuments() for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetIndexedDocuments() throws Exception {
        im.createDocumentIndex(TestHelper.getMediumWordSource());
        im.createDocumentIndex(TestHelper.getSimpleWordSource());

        Set documents = persistence.getIndexedDocuments();

        assertEquals("The returned set should contain two elements.", 2,
            documents.size());
    }
}
